mod ssh;
mod console;
mod dokku;

use colorize::AnsiColor;
use structopt::StructOpt;
use structopt::clap::AppSettings;

/*
 1 - ask for project name
 2 - dokku apps:list | grep project name ( check for conflicts)
 2.1
   - ask for mysql or mariadb or postgres
   - ask for redis
   - ask for domain names
 3 - dokku apps:create <project name>
 4 - dokku mysql|mariadb|postgres:create <project name>
 5 - dokku mysql|mariadb|postgres:link <project name> <project name>
 6 - setup storage dokku storage:mount <project name>
 7 - check for proxy ports (should be http:80:80) dokku proxy:report <project name>
 8 - do steps 2-7 for <project name>-dev

 */

#[derive(Debug, StructOpt)]
#[structopt(name = "mariposa", about = "generate a whole project on dokku", long_about="Use -s to provide a server. Use -i to provide a key file.", global_settings = &[AppSettings::ColoredHelp])]
struct Args {
    #[structopt(short = "s", long, default_value = "prod.fixrs.dev", help="Use -s to provide a server. Defaults to prod.fixrs.dev.")]
    host: String,
    #[structopt(short = "i", long, default_value = "~/.ssh/id_rsa", help="Use -i to provider a key file. Defaults to ~/.ssh/id_rsa")]
    private_key_path: String,
}


fn main() {
    let project_name = console::question(String::from("Enter the project name : "));
    if project_name.len() < 2 {
        panic!("Please enter a longer application name.")
    }
    let domains = console::question(String::from("What domain names would you like to bind to it (add a space between each if more than one is required)? "));

    let project_prod = project_name.to_owned() + "-production";
    let project_dev = project_name.to_owned() + "-develop";

    let mut projects: [&String; 2] = [&project_dev, &project_prod];

    let need_mysql = console::question_yn(format!("Do you need a mysql server? (y|{}) : ", "n".underlined()));
    let need_postgres = console::question_yn(format!("Do you need a postgres server? (y|{}) : ", "n".underlined()));
    let need_redis = console::question_yn(format!("Do you need a redis server? (y|{}) : ", "n".underlined()));
    let wp_app = console::question_ny(format!("Is this a FIXRS wordpress app (with uploads at /var/www/html/wp-content/uploads)? ({}|n) : ", "y".underlined()));

    for (_i, project) in projects.iter_mut().enumerate() {
        let project_exists = dokku::check_project_exists(&project);
        if project_exists {
            let project_bold = project.to_owned().bold().red();
            println!("This server already has a project named {}.", project_bold);
            std::process::exit(1);
        }

        dokku::create_service(dokku::ServiceType::APP, &project);
        if need_mysql {
            dokku::create_service(dokku::ServiceType::MYSQL, &project);
            dokku::link_service(dokku::ServiceType::MYSQL, &project);
        }
        if need_postgres {
            dokku::create_service(dokku::ServiceType::POSTGRES, &project);
            dokku::link_service(dokku::ServiceType::MYSQL, &project);
        }
        if need_redis {
            dokku::create_service(dokku::ServiceType::REDIS, &project);
            dokku::link_service(dokku::ServiceType::REDIS, &project);
        }
        if wp_app {
            dokku::mount(&project);
        }

        println!("Forcing ports on the new application...");
        dokku::force_ports(&project);
        if domains != "" {
            dokku::set_domains(&project, &domains);
        }

    }

}
